{#
Copyright (C) 2020 CERN.
Copyright (C) 2020 Northwestern University.
Copyright (C) 2024 Graz University of Technology.

Invenio-Curations is free software; you can redistribute it and/or modify it
under the terms of the MIT License; see LICENSE file for more details.
#}
{%- set title = _("Requests") %}
{%- extends "invenio_app_rdm/users/base.html" %}

{% set active_dashboard_menu_item = 'curation-overview' %}

{%- block javascript %}
{{ super() }}
{{ webpack['invenio-app-rdm-user-requests.js'] }}
{%- endblock javascript %}

{%- block user_dashboard_body %}

<div id="invenio-search-config"
  data-invenio-search-config='{{ search_app_curations_requests_config(app_id="InvenioAppRdm.DashboardRequests") | tojson }}'>
</div>

{%- endblock user_dashboard_body %}
