{% set curation_request = notification.context.request %}
{% set group = curation_request.receiver %}
{% set creator = curation_request.created_by %}
{% set record = curation_request.topic %}
{% set request_id = curation_request.id %}
{% set executing_user = notification.context.executing_user %}
{% set message = notification.context.message | safe if notification.context.message else '' %}
{% set record_title = record.metadata.title %}
{% set curator_name = executing_user.profile.full_name %}
{% if not curator_name  %} 
    {% set curator_name = executing_user.email %}
{% else %} 
    {% set curator_name = curator_name +  " - " + executing_user.email %} 
{% endif %}

{# TODO: use request.links.self_html when issue issue is resolved: https://github.com/inveniosoftware/invenio-rdm-records/issues/1327 #}
{% set request_link = "{ui}/me/requests/{id}".format(
    ui=config.SITE_UI_URL, id=request_id
    )
%}
{% set account_settings_link = "{ui}/account/settings/notifications".format(
    ui=config.SITE_UI_URL
    )
%}

{%- block subject -%}
       {{ _("👁️ Review started for '{record_title}'").format(record_title=record_title) }}
{%- endblock subject -%}

{%- block html_body -%}
    <table style="font-family:'Lato',Helvetica,Arial,sans-serif;border-spacing:15px">
        <tr>
            <td>{{ _("The metadata curator '{curator_name}' started reviewing the record '{record_title}'").format(curator_name=curator_name, record_title=record_title) }}
            {% if message %}
            {{ _(" with the following message:")}}
            {% endif %}
            </td>
        </tr>
        <tr>
            {% if message %}
            <td><em>{{message}}</em></td>
            {% endif %}
        </tr>
        <tr>
            <td><a href="{{ request_link }}" class="button">{{ _("Check out the curation request")}}</a></td>
        </tr>
        <tr>
            <td><strong>_</strong></td>
        </tr>
        <tr>
            <td style="font-size:smaller">{{ _("This is an auto-generated message. To manage notifications, visit your")}} <a href="{{account_settings_link}}">{{ _("account settings")}}</a>.</td>
        </tr>
    </table>
{%- endblock html_body %}

{%- block plain_body -%}
{{ _("The metadata curator {curator_name} started reviewing the record '{record_title}'").format(curator_name=curator_name, record_title=record_title) }}

{% if message %}
{{ _("with the following message:")}}
{{message}}
{% endif %}

[{{ _("Check out the curation request") }}]({{ request_link }})

{{ _("This is an auto-generated message. To manage notifications, visit your account settings")}}
{%- endblock plain_body %}

{# Markdown for Slack/Mattermost/chat #}
{%- block md_body -%}
{{ _("The metadata curator *{curator_name}* started reviewing the record *{record_title}*").format(curator_name=curator_name, record_title=record_title) }}

{% if message %}
{{ _("with the following message:")}}
{{message}}
{% endif %}

[{{ _("Check out the curation request") }}]({{ request_link }})

{{ _("This is an auto-generated message. To manage notifications, visit your account settings")}}
{%- endblock md_body %}
